@@ -13,13 +13,13 @@ import xml.etree.ElementTree as ET
 try:
     from urllib.request import urlopen, urlretrieve
 except ImportError:
-        from urllib import urlopen, urlretrieve
+    from urllib import urlopen, urlretrieve
 import argparse
-version = """ stringMLST v0.5.1.1 (updated : January 5, 2018) """
+version = """ stringMLST v0.6.3 (updated : September 02, 2020) """
 """
 
-stringMLST free for academic users and requires permission before any commercial 
-use for any version of this code/algorithm. If you are a commercial user, please 
+stringMLST free for academic users and requires permission before any commercial
+use for any version of this code/algorithm. If you are a commercial user, please
 contact king.jordan@biology.gatech.edu for permissions
 
 LICENSE TERMS FOR stringMLST
@@ -256,54 +256,63 @@ predict part starts here
 # Description: Gets the URLs from pubMLST for the required
 #              files (alleles, profile)
 #############################################################
-def get_links(speciesName,schemes):
-    global loci
-    loci = {}
-    URL="http://pubmlst.org/data/dbases.xml"
-    xml = urlopen(URL)
-    tree = ET.parse(xml)
-    root = tree.getroot()
-    for species in root:
-        if re.search(schemes[speciesName], species.text,re.IGNORECASE):
+def get_links(xmlData, savePath, speciesName):
+    lociList = {}
+    profileURL = None
+    for species in xmlData:
+        if re.search(re.escape(speciesName), species.text, re.IGNORECASE, ):
             for mlst in species:
                 for database in mlst:
                     for child in database:
                         if child.tag == "profiles":
-                            profileURL = child[1].text
+                            profileURL = child[0].text
                         if child.tag == "loci":
                             for locus in child:
-                                loci[locus.text.rstrip()] = locus[0].text
- 
-    return profileURL
+                                lociList[locus.text.rstrip()] = locus[0].text
+    if profileURL is None:
+        profileError = "Parsing failed: could not find profiles file"
+        print(profileError)
+        print("This usually means the provided species, '{}', does not exist on PubMLST".format(speciesName))
+        print("Use `{} --getMLST --species list` to list available species".format(sys.argv[0]))
+        print("Or visit PubMLST for more information:\nhttps://pubmlst.org/data/")
+        logging.debug(profileError)
+        sys.exit(1)
+    elif lociList == {}:
+        lociError = "Parsing failed: could not find allele sequences"
+        logging.debug(lociError)
+        print(lociError)
+        sys.exit(1)
+    else:
+        return profileURL, lociList
 #############################################################
 # Function   : get_files
 # Input      : URLs from get_links
 # Output     : Downloads files and builds database
 #############################################################
-def get_files(profileURL,species):
+def get_files(filePrefix, loci, profileURL, speciesName):
     with open(config, "w") as configFile:
         configFile.write("[loci]\n")
         for file in loci:
             localFile = filePrefix + "_" + file + ".tfa"
             try:
-                localFfile, headers = urlretrieve(loci[file],localFile)
+                localFile, headers = urlretrieve(loci[file], localFile)
             except:
-                print( '\033[91m' + "There was an error downloading " + file + '\033[0m')
-                pass 
+                print('\033[91m' + "There was an error downloading " + file + '\033[0m')
+                pass
             configFile.write(file + "\t" + filePrefix + "_" + file + ".tfa\n")
         localFile = filePrefix + "_profile.txt"
-        localFile, headers = urlretrieve(profileURL,localFile)
+        localFile, headers = urlretrieve(profileURL, localFile)
         configFile.write("[profile]\n")
         configFile.write("profile\t" + filePrefix + "_profile.txt\n")
         configFile.close()
         try:
-            makeCustomDB(config,k,filePrefix)
+            makeCustomDB(config, k, filePrefix)
         except:
-            print( '\033[91m' + "Failed to create database " + schemes[species] + '\033[0m' )
+            print('\033[91m' + "Failed to create database " + speciesName + '\033[0m')
             pass
         else:
-            print("\t" + '\033[92m' + "Database ready for " + schemes[species] + '\033[0m')
-            print( "\t" + filePrefix)
+            print("\t" + '\033[92m' + "Database ready for " + speciesName + '\033[0m')
+            print("\t" + filePrefix)
 ############################################################
 # Function   : batchTool
 # Input      : Directory name, paired or single, k value
@@ -733,10 +742,10 @@ def loadWeightDict(weightFile):
         for line in lines:
             array = line.rstrip().rsplit('\t')
             try:
-               (loc, allele) =  array[0].replace('-','_').rsplit('_',1)
+               (loc, allele) = array[0].replace('-', '_').rsplit('_', 1)
             except ValueError:
                 print("Error : Allele name in locus file should be seperated by '_' or '-'")
-                exit(0) 
+                exit(0)
             if loc not in weightDict:
                 weightDict[loc] = {}
             weightDict[loc][allele] = float(array[1])
@@ -787,7 +796,7 @@ def getCoverage(results):
             with open(bed, 'w') as bedFile:
                 for gene in configDict['loci']:
                     genes = Fasta(configDict['loci'][gene])
-                    allele = gene+'_'+re.sub("*", "", str(results[sample][gene]))
+                    allele = gene+'_'+re.sub('\*', "", str(results[sample][gene]))
                     tmpFasta.write('>'+gene+'\n')
                     bedFile.write(gene+'\t0\t'+str(len(genes[allele]))+'\n')
                     for line in genes[allele]:
@@ -902,11 +911,11 @@ def formKmerDB(configDict, k, output_filename):
             sum += l
             n += 1
             try:
-                (loc, num) =  allele.replace('-','_').rsplit('_',1)
+                (loc, num) = allele.replace('-', '_').rsplit('_', 1)
             except ValueError:
                 print("Error : Allele name in locus file should be seperated by '_' or '-'")
-                exit(0) 
-            splitId =  allele.replace('-','_').rsplit('_',1)
+                exit(0)
+            splitId = allele.replace('-', '_').rsplit('_', 1)
             i = 0
             while i+k <= l:
                 kmer = seq[i:i+k]
@@ -1177,8 +1186,8 @@ Required arguments
 --getMLST
     Identifier for getMLST module
 --species= <species name>
-    Species name from the pubMLST schemes (use --schemes to get list of available schemes)
-    "all" will download and build all 
+    Species name from the pubMLST schemes (use "--species show" to get list of available schemes)
+    "all" will download and build all
 Optional arguments
 -k = <kmer length>
     Kmer size for which the db has to be formed(Default k = 35). Note the tool works best with kmer length in between 35 and 66
@@ -1316,8 +1325,9 @@ Required arguments
 --getMLST
     Identifier for getMLST module
 --species= <species name>
-    Species name from the pubMLST schemes (use --schemes to get list of available schemes)
-    "all" will download and build all 
+    Species name from the pubMLST schemes
+    Use "show" or "list" to list available schemes
+    "all" will download and build all available schemes
 Optional arguments
 -k = <kmer length>
     Kmer size for which the db has to be formed(Default k = 35). Note the tool works best with kmer length in between 35 and 66
@@ -1333,132 +1343,6 @@ Optional arguments
 =============================================================================================
 
 """
-schemes = {"achromobacter" : "Achromobacter spp.",
-"baumannii1" : "Acinetobacter baumannii#1",
-"baumannii2" : "Acinetobacter baumannii#2",
-"aeromonas" : "Aeromonas spp.",
-"anaplasma-phagocytophilum" : "Anaplasma phagocytophilum",
-"arcobacter" : "Arcobacter spp.",
-"aspergillus-fumigatus" : "Aspergillus fumigatus",
-"bacillus-cereus" : "Bacillus cereus",
-"bacillus-licheniformis" : "Bacillus licheniformis",
-"bacillus-subtilis" : "Bacillus subtilis",
-"bartonella-henselae" : "Bartonella henselae",
-"bordetella" : "Bordetella spp.",
-"borrelia" : "Borrelia spp.",
-"brachyspira-hampsonii" : "Brachyspira hampsonii",
-"brachyspira-hyodysenteriae" : "Brachyspira hyodysenteriae",
-"brachyspira-intermedia" : "Brachyspira intermedia",
-"brachyspira-pilosicoli" : "Brachyspira pilosicoli",
-"brachyspira" : "Brachyspira spp.",
-"brucella" : "Brucella spp.",
-"burkholderia-cepacia-complex" : "Burkholderia cepacia complex",
-"burkholderia-pseudomallei" : "Burkholderia pseudomallei",
-"campylobacter-concisus-curvus" : "Campylobacter concisus/curvus",
-"campylobacter-fetus" : "Campylobacter fetus",
-"campylobacter-helveticus" : "Campylobacter helveticus",
-"campylobacter-hyointestinalis" : "Campylobacter hyointestinalis",
-"campylobacter-insulaenigrae" : "Campylobacter insulaenigrae",
-"campylobacter-jejuni" : "Campylobacter jejuni",
-"campylobacter-lanienae" : "Campylobacter lanienae",
-"campylobacter-lari" : "Campylobacter lari",
-"campylobacter-sputorum" : "Campylobacter sputorum",
-"campylobacter-upsaliensis" : "Campylobacter upsaliensis",
-"candida-albicans" : "Candida albicans",
-"candida-glabrata" : "Candida glabrata",
-"candida-krusei" : "Candida krusei",
-"candida-tropicalis" : "Candida tropicalis",
-"carnobacterium-maltaromaticum" : "Carnobacterium maltaromaticum",
-"chlamydiales" : "Chlamydiales spp.",
-"citrobacter-freundii" : "Citrobacter freundii",
-"clonorchis-sinensis" : "Clonorchis sinensis",
-"clostridium-botulinum" : "Clostridium botulinum",
-"clostridium-difficile" : "Clostridium difficile",
-"clostridium-septicum" : "Clostridium septicum",
-"corynebacterium-diphtheriae" : "Corynebacterium diphtheriae",
-"cronobacter" : "Cronobacter spp.",
-"enterobacter-cloacae" : "enterobacter cloacae",
-"enterococcus-faecalis" : "enterococcus faecalis",
-"enterococcus-faecium" : "enterococcus faecium",
-"ecoli1" : "escherichia coli#1",
-"ecoli2" : "escherichia coli#2",
-"flavobacterium-psychrophilum" : "Flavobacterium psychrophilum",
-"haemophilus-influenzae" : "Haemophilus influenzae",
-"haemophilus-parasuis" : "Haemophilus parasuis",
-"helicobacter-cinaedi" : "Helicobacter cinaedi",
-"helicobacter-pylori" : "Helicobacter pylori",
-"helicobacter-suis" : "Helicobacter suis",
-"kingella-kingae" : "Kingella kingae",
-"klebsiella-oxytoca" : "Klebsiella oxytoca",
-"klebsiella-pneumoniae" : "Klebsiella pneumoniae",
-"kudoa-septempunctata" : "Kudoa septempunctata",
-"lactobacillus-salivarius" : "Lactobacillus salivarius",
-"leptospira" : "Leptospira spp.",
-"leptospira2" : "Leptospira spp.#2",
-"leptospira3" : "Leptospira spp.#3",
-"listeria" : "Listeria monocytogenes",
-"macrococcus-canis" : "Macrococcus canis",
-"macrococcus-caseolyticus" : "Macrococcus caseolyticus",
-"mannheimia-haemolytica" : "Mannheimia haemolytica",
-"melissococcus-plutonius" : "Melissococcus plutonius",
-"moraxella" : "Moraxella catarrhalis",
-"mycobacterium-abscessus" : "Mycobacterium abscessus",
-"mycobacterium-massiliense" : "Mycobacterium massiliense",
-"mycoplasma-agalactiae" : "Mycoplasma agalactiae",
-"mycoplasma-bovis" : "Mycoplasma bovis",
-"mycoplasma-hyopneumoniae" : "Mycoplasma hyopneumoniae",
-"mycoplasma-hyorhinis" : "Mycoplasma hyorhinis",
-"mycoplasma-iowae" : "Mycoplasma iowae",
-"mycoplasma-pneumoniae" : "Mycoplasma pneumoniae",
-"mycoplasma-synoviae" : "Mycoplasma synoviae",
-"neisseria" : "Neisseria spp.",
-"orientia-tsutsugamushi" : "Orientia tsutsugamushi",
-"ornithobacterium-rhinotracheale" : "Ornithobacterium rhinotracheale",
-"paenibacillus-larvae" : "Paenibacillus larvae",
-"pasteurella-multocida1" : "Pasteurella multocida#1",
-"pasteurella-multocida2" : "Pasteurella multocida#2",
-"pediococcus-pentosaceus" : "Pediococcus pentosaceus",
-"photobacterium-damselae" : "Photobacterium damselae",
-"porphyromonas-gingivalis" : "Porphyromonas gingivalis",
-"propionibacterium-acnes" : "Propionibacterium acnes",
-"pseudomonas-aeruginosa" : "Pseudomonas aeruginosa",
-"pseudomonas-fluorescens" : "Pseudomonas fluorescens",
-"riemerella-anatipestifer" : "Riemerella anatipestifer",
-"salmonella-enterica" : "Salmonella enterica",
-"sinorhizobium" : "Sinorhizobium spp.",
-"s.aureus" : "Staphylococcus aureus",
-"s.epidermidis" : "Staphylococcus epidermidis",
-"s.haemolyticus" : "Staphylococcus haemolyticus",
-"s.lugdunensis" : "Staphylococcus lugdunensis",
-"s.pseudintermedius" : "Staphylococcus pseudintermedius",
-"stapylococcus-hominis" : "Stapylococcus hominis",
-"stenotrophomonas-maltophilia" : "Stenotrophomonas maltophilia",
-"s.agalactiae" : "Streptococcus agalactiae",
-"s.canis" : "Streptococcus canis",
-"s.dysgalactiae-equisimilis" : "Streptococcus dysgalactiae equisimilis",
-"s.gallolyticus" : "Streptococcus gallolyticus",
-"s.oralis" : "Streptococcus oralis",
-"s.pneumoniae" : "Streptococcus pneumoniae",
-"s.pyogenes" : "Streptococcus pyogenes",
-"s.suis" : "Streptococcus suis",
-"s.thermophilus" : "Streptococcus thermophilus",
-"s.thermophilus2" : "Streptococcus thermophilus#2",
-"s.uberis" : "Streptococcus uberis",
-"s.zooepidemicus" : "Streptococcus zooepidemicus",
-"streptomyces-spp" : "Streptomyces spp",
-"taylorella" : "Taylorella spp.",
-"tenacibaculum" : "Tenacibaculum spp.",
-"trichomonas-vaginalis" : "Trichomonas vaginalis",
-"vibrio-cholerae" : "Vibrio cholerae",
-"vibrio-parahaemolyticus" : "Vibrio parahaemolyticus",
-"vibrio" : "Vibrio spp.",
-"vibrio-tapetis" : "Vibrio tapetis",
-"vibrio-vulnificus" : "Vibrio vulnificus",
-"wolbachia" : "Wolbachia",
-"xylella-fastidiosa" : "Xylella fastidiosa",
-"yersinia-pseudotuberculosis" : "Yersinia pseudotuberculosis",
-"yersinia-ruckeri" : "Yersinia ruckeri",
-"yersinia" : "Yersinia spp."}
 
 """The Program Starts Execution Here"""
 """Default Params"""
@@ -1483,7 +1367,7 @@ log = ''
 k = 35
 fuzzy = 300
 coverage = False
-#print 'ARGV      :', sys.argv[1:]
+#print'ARGV      :', sys.argv[1:]
 #exit(0)
 """Input arguments"""
 options, remainder = getopt.getopt(sys.argv[1:], 'o:x1:2:k:l:bd:pshP:c:trva:z:C', [
@@ -1564,7 +1448,7 @@ for opt, arg in options:
             print("You provided '" + arg + "' for your fuzziness threshold, which is not an integer value")
             exit(0)
     elif opt in '--schemes':
-        print (", ".join(sorted(schemes.keys())))
+        print("The `--schemes` option has been depreciated.  Please use `--species list` to see available schemes")
         exit(0)
     elif opt in '--getMLST':
         downloadDB = True
@@ -1613,51 +1497,70 @@ elif predict is True:
         getCoverage(results)
     printResults(results, output_filename, overwrite, timeDisp)
 elif downloadDB is True:
-    global filePrefix
+    dbURL = "http://pubmlst.org/data/dbases.xml"
+    databaseXML = urlopen(dbURL)
+    dbTree = ET.parse(databaseXML)
+    dbRoot = dbTree.getroot()
     if species is None:
-        print ("Please refer to --help to more information")
+        print("Please refer to --help to more information")
         print()
-        print ("Expected command format:")
+        print("Expected command format:")
         print("stringMLST.py --getMLST --species= <species> [-k kmer length] [-P DB prefix]")
-        print ()
-        print("Available MLST Schemes:")
-        print (", ".join(sorted(schemes.keys())))
+        print()
+        print("To printavailable MLST Schemes use:")
+        print("stringMLST.py --getMLST --species show")
         exit(0)
+    elif species == "show" or species == "list":
+        for species in dbRoot:
+            print(species.text.rstrip())
     elif species == "all":
         print("Using a kmer size of " + str(k) + " for all databases.")
-        for key in sorted(schemes.keys()):
-            filePrefix = str(dbPrefix.rsplit("/",1)[0]) + "/" + key
-            print ('\033[1m' + "Preparing: " + schemes[key] + '\033[0m') 
-            # Move the rest of this informational message into the download handler 
+        for species in dbRoot:
+            speciesName = species.text.rstrip()
+            print('\033[1m' + "Preparing: " + speciesName + '\033[0m')
+            if re.search('[/#. ()]', speciesName):
+                normSpeciesName = re.sub('[/# ]', "_", speciesName)
+                normSpeciesName = re.sub('[.()]', "", normSpeciesName)
+                print('\t\033[33m' + "INFO: normalizing name to: " + normSpeciesName + '\033[0m')
+            else:
+                normSpeciesName = speciesName
+            filePrefix = str(dbPrefix.rsplit("/", 1)[0]) + "/" + normSpeciesName
+            # Move the rest of this informational message into the download handler
             # + " ( " + filePrefix + "/" + key + "_" +str(k) + " )")
             try:
                 os.makedirs(filePrefix)
             except OSError:
                pass
-            filePrefix = filePrefix + "/" + key
-            config = filePrefix + "_config.txt" 
-            profileURL = get_links(key,schemes)
-            get_files(profileURL,key)
+            filePrefix = filePrefix + "/" + normSpeciesName
+            config = filePrefix + "_config.txt"
+            profileURL, loci = get_links(dbRoot, filePrefix, speciesName)
+            get_files(filePrefix, loci, profileURL, speciesName)
     else:
+        print('\033[1m' + "Preparing: " + species + '\033[0m')
+        if re.search('[/#. ()]', species):
+            normSpeciesName = re.sub('[/# ]', "_", species)
+            normSpeciesName = re.sub('[.()]', "", normSpeciesName)
+            print('\t\033[33m' + "INFO: normalizing name to: " + normSpeciesName + '\033[0m')
+        else:
+            normSpeciesName = species
         try:
-            os.makedirs(dbPrefix.rsplit("/",1)[0])
+            os.makedirs(dbPrefix.rsplit("/", 1)[0])
         except OSError:
             pass
         if len(re.findall("/", dbPrefix)) == 0:
-            filePrefix = dbPrefix + "/" + species
-        elif len(re.findall("/", dbPrefix)) == 1 and len(dbPrefix.rsplit("/",1)[1]) > 0:
+            filePrefix = dbPrefix + "/" + normSpeciesName
+        elif len(re.findall("/", dbPrefix)) == 1 and len(dbPrefix.rsplit("/", 1)[1]) > 0:
             filePrefix = dbPrefix
-        elif len(re.findall("/", dbPrefix)) == 1 and len(dbPrefix.rsplit("/",1)[1]) == 0:
-            filePrefix = dbPrefix + species
+        elif len(re.findall("/", dbPrefix)) == 1 and len(dbPrefix.rsplit("/", 1)[1]) == 0:
+            filePrefix = dbPrefix + normSpeciesName
         elif len(re.findall("/", dbPrefix)) > 1:
             if dbPrefix.endswith('/'):
-                filePrefix = dbPrefix + species
+                filePrefix = dbPrefix + normSpeciesName
             else:
                 filePrefix = dbPrefix
         config = filePrefix + "_config.txt"
-        profileURL = get_links(species,schemes)
-        get_files(profileURL,species)
-
+        profileURL, loci = get_links(dbRoot, filePrefix, species)
+        get_files(filePrefix, loci, profileURL, species)
 else:
     print(helpTextSmall)
     print("Error: Please select the mode: buildDB (for database building) or predict (for ST discovery) module")
